using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Demo.Properties;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.DataBinding.ConditionalDataBinding
{
    [Example("Conditional data binding", "Data binding")]
    public partial class ConditionalDataBindingForm : UserControl
    {
        #region Fields

        private readonly BindingList<Deal> _allDeals = new BindingList<Deal>();
        private readonly BindingList<Order> _allOrders = new BindingList<Order>();
        private readonly BindingList<Basket> _baskets = new BindingList<Basket>();
        private readonly Random _r = new Random();
        private readonly BindingList<Order> _singleOrders = new BindingList<Order>();

        #endregion

        public ConditionalDataBindingForm()
        {
            InitializeComponent();

            grid1.DataSource = new MultipleDataSources(new IList[] {_baskets, _singleOrders});
            grid2.DataSource = new MultipleDataSources(new IList[] {_baskets, _singleOrders});


            grid2.Filter = new Filter(OnRowFilter);

            //Populate with random data
            Populate();
        }

        #region Private event handlers

        private void OnGridPaintCell(object sender, PaintCellEventArgs e)
        {
            if (e.Cell.Column != null && e.Cell.Column.Id == "Id")
            {
                e.ImageSettings.Alignment = ContentAlignment.MiddleLeft;

                if (e.Cell.Row.DataObject is Basket)
                {
                    e.Image = Resources.folder_cubes;
                }
                if (e.Cell.Row.DataObject is Order)
                {
                    e.Image = Resources.document_pulse;
                }
                if (e.Cell.Row.DataObject is Deal)
                {
                    e.Image = Resources.bullet_ball_glass_blue;
                }    
            }

            
        }

        private void OnGridPaintRow(object sender, PaintRowEventArgs e)
        {
            if(e.Row.DataObject is Basket)
            {
                e.Appearance.BackColor = Color.FromArgb(253, 241, 219);
                e.Appearance.GradientEndBackColor = Color.FromArgb(241, 231, 200);
                e.Appearance.GradientEnabled = true;
            }
            if(e.Row.DataObject is Order)
            {
                e.Appearance.BackColor = Color.AliceBlue;
                e.Appearance.GradientEndBackColor = Color.FromArgb(214, 234, 254);
                e.Appearance.GradientEnabled = true;
            }
            if (e.Row.DataObject is Deal)
            {
                e.Appearance.BackColor = Color.FromArgb(241, 253, 219);
                e.Appearance.GradientEndBackColor = Color.FromArgb(231, 241, 200);
                e.Appearance.GradientEnabled = true;

                //PaintHelper.AlphaBlend(e.Appearance, Color.FromArgb(30, Color.Green));
            }
        }

        private void OnRowExpansionChanging(object sender, GridRowExpansionEventArgs e)
        {
            Cell cell = e.Grid.HitTests.CellTest(e.Grid.PointToClient(Cursor.Position));
            if (cell != null && cell.Column != null && cell.Column.Id == "Status")
            {
                e.Cancel = true;
            }
        }

        private void OnGrid1RowAdding(object sender, GridRowAddingEventArgs e)
        {
            Basket basket = e.DataObject as Basket;
            if (basket != null)
            {
                e.DataObject = basket.Orders;
                e.InsertionType = InsertionType.AsChildAndCollection;
            }
        }

        private void OnGrid2RowAdding(object sender, GridRowAddingEventArgs e)
        {
            Basket basket = e.DataObject as Basket;
            if (basket != null)
            {
                e.DataObject = basket.Orders;
                e.InsertionType = !cbSkipBaskets.Checked
                                      ? InsertionType.AsChildAndCollection
                                      : InsertionType.AsCollection;
            }
            Order order = e.DataObject as Order;
            if (order != null)
            {
                e.DataObject = order.Deals;
                e.InsertionType = !btnSkipOrders.Checked
                                      ? InsertionType.AsChildAndCollection
                                      : InsertionType.AsCollection;
            }
        }

        private void OnExpandRowsAfterAdding(object sender, GridRowEventArgs e)
        {
            e.Row.Expanded = true;
        }

        private void OnAddBasket(object sender, EventArgs e)
        {
            _baskets.Add(new Basket("Basket #" + (_baskets.Count + 1)));
        }

        private void OnAddSingleOrder(object sender, EventArgs e)
        {
            Order order = new Order("Order #" + (_allOrders.Count + 1),
                                    _r.Next()%2 == 0 ? Way.Buy : Way.Sell,
                                    _r.Next(50, 150) + _r.NextDouble(),
                                    _r.Next(100, 50000),
                                    "Product #" + _r.Next(1, 100),
                                    null);
            _allOrders.Add(order);
            _singleOrders.Add(order);
        }

        private void OnAddOrderToBasket(object sender, EventArgs e)
        {
            Basket basket = _baskets.Count > 0 ? _baskets[_r.Next(_baskets.Count)] : null;
            if (basket != null)
            {
                Order order = new Order("Order #" + (_allOrders.Count + 1),
                                        _r.Next()%2 == 0 ? Way.Buy : Way.Sell,
                                        _r.Next(50, 150) + _r.NextDouble(),
                                        _r.Next(100, 50000),
                                        "Product #" + _r.Next(1, 100),
                                        basket);
                _allOrders.Add(order);
                basket.AddOrder(order);
            }
        }

        private void OnAddDeal(object sender, EventArgs e)
        {
            Order order = _allOrders.Count > 0 ? _allOrders[_r.Next(_allOrders.Count)] : null;
            if (order != null)
            {
                Deal deal = new Deal(order,
                                     "Deal #" + (_allDeals.Count + 1),
                                     _r.Next(50, 150) + _r.NextDouble(),
                                     _r.Next(50, 1000),
                                     "Counterpart " + _r.Next(1, 100));

                _allDeals.Add(deal);
                order.AddDeal(deal);
            }
        }

        private void OnRebindGrid2DataSources(object sender, EventArgs e)
        {
            //Rebind data sources to grid2
            grid2.DataSource = new MultipleDataSources(new IList[] {_baskets, _singleOrders});
        }

        private void OnCustomEditor(object sender, GridCellEditableEventArgs e)
        {
            if (e.Cell.Row.DataObject is Basket && e.Cell.Column.Id == "Status")
            {
                //Basket doesn't have Status setter. Force the cell editable
                e.Editable = true;
            }
        }

        private void OnGridSelectionChanged(object sender, GridRowEventArgs e)
        {
            grid2.FilterRefresh();
        }

        private void OnFilterCheckedChanged(object sender, EventArgs e)
        {
            grid2.FilterRefresh();
        }

        private bool OnRowFilter(Row row)
        {
            if(cbFilter.Checked)
            {
                if (grid1.Selection.Count == 0)
                {
                    return false;
                }

                Basket basket = row.DataObject as Basket;
                if (basket != null)
                {
                    //Check if the basket in the upper grid is selected
                    Row basketRow = grid1.DataObjects.FindFirstRow(basket);

                    //Check if any of basket children in the grid1 is selected
                    bool filtered = basketRow != null && !basketRow.Selected;
                    if (filtered)
                    {
                        foreach (Row child in basketRow.VisibleChildren)
                        {
                            filtered &= !child.Selected;
                            if(!filtered)
                            {
                                break;
                            }
                        }    
                    }

                    return filtered;
                }

                Order order = row.DataObject as Order;
                Deal deal = row.DataObject as Deal;

                order = order ?? (deal != null ? deal.Order : null);

                if (order != null)
                {
                    //Check if the basket in the upper grid is selected
                    Row orderRow = grid1.DataObjects.FindFirstRow(order);
                    bool filtered = orderRow != null && !orderRow.Selected;
                    if (filtered && order.Basket != null)
                    {
                        //Check the parent basket
                        Row basketRow = orderRow.Parent;
                        filtered = basketRow != null && !basketRow.Selected;

                    }
                    return filtered;
                }

                
            }
            return false;
        }

        #endregion

        #region Private methods

        private void Populate()
        {
            for (int i = 0; i < 3; ++i)
            {
                OnAddBasket(this, EventArgs.Empty);
            }

            for (int i = 0; i < 8; ++i)
            {
                OnAddOrderToBasket(this, EventArgs.Empty);
            }

            for (int i = 0; i < 2; ++i)
            {
                OnAddSingleOrder(this, EventArgs.Empty);
            }

            for (int i = 0; i < 15; ++i)
            {
                OnAddDeal(this, EventArgs.Empty);
            }
        }

        #endregion
    }
}