using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;
using Appearance = Dapfor.Net.Ui.Appearance;

namespace Dapfor.Net.Demo.Examples.DataBinding.Populating
{
    [Example("How to populate .Net Grid", "Data binding")]
    public partial class ExampleForm : UserControl
    {
        #region Constructor

        public ExampleForm()
        {
            InitializeComponent();

            rbDataObject.Checked = true;
        }

        #endregion

        private void AddingOneByOne()
        {
            //Remove all previous rows and headers
            ClearData();

            //Create two headers (for top-level items and theirs children)
            Header header1 = Header.FromDataType(typeof (RootClass));
            Header header2 = Header.FromDataType(typeof (ChildClass));
            grid.Headers.Add(header1);
            grid.Headers.Add(header2);

            SetAppearance();
            grid.Hierarchy.ButtonBehaviour = ExpansionButtonBehaviour.HideWhenNoChildren;

            Random random = new Random();

            //Populate with some data
            for (int i = 0; i < 20; ++i)
            {
                //Populate with top-level items
                RootClass rootObject = new RootClass(random.NextDouble(),
                                                     random.Next(5),
                                                     "Item " + i,
                                                     random.Next()%2 == 0 ? RootClass.EnumExample.Internal : RootClass.EnumExample.External);
                //Add to the grid
                Row rootRow = grid.Rows.Add(rootObject);

                //Add ten children to each root
                for (int j = 0; j < 10; ++j)
                {
                    ChildClass childObject = new ChildClass(DateTime.Now + new TimeSpan(1000000000000*(random.Next()%1000)),
                                                            random.Next(5),
                                                            new TimeSpan(random.Next()));
                    //Add as a child
                    rootRow.Add(childObject);
                }
            }

            grid.Nodes[3].Expanded = true;
        }

        private void AddingIDictionaryObjects()
        {
            //Remove all previous rows and headers
            ClearData();

            //Root header
            Header header1 = new Header();
            grid.Headers.Add(header1);
            header1.Add(new Column("Key [String]"));
            header1.Add(new Column("Key [DateTime]"));
            header1.Add(new Column("Key [Int]"));
            header1.Add(new Column("Key [bool]"));

            //Child header
            Header header2 = new Header();
            grid.Headers.Add(header2);
            header2.Add(new Column("Key [TimeSpan]"));
            header2.Add(new Column("Key [decimal]"));

            //Set header appearance
            SetAppearance();
            grid.Hierarchy.ButtonBehaviour = ExpansionButtonBehaviour.HideWhenNoChildren;

            Random random = new Random();

            for (int i = 0; i < 20; ++i)
            {
                //Create root item. The keys of the hash table correspond to the columns, 
                //and values correspond to the data, displayed in cells
                Hashtable rootItem = new Hashtable();
                rootItem.Add("Key [String]", "Root item " + i);
                rootItem.Add("Key [DateTime]", DateTime.Now + new TimeSpan(1000000000000 * (random.Next() % 1000)));
                rootItem.Add("Key [Int]", random.Next(5));
                rootItem.Add("Key [bool]", i % 2 == 0);

                Row rootRow = grid.Rows.Add(rootItem);
                rootRow.Expanded = false;


                //Add some children (first field is Int32 and the second is String)
                for (int j = 0; j < 10; ++j)
                {
                    Hashtable childItem = new Hashtable();
                    childItem.Add("Key [TimeSpan]", new TimeSpan(random.Next() % 10000000));
                    childItem.Add("Key [decimal]", Decimal.Round((Decimal)random.NextDouble(), 4));
                    rootRow.Add(childItem);
                }
            }

            grid.Nodes[3].Expanded = true;
            grid.Nodes[8].Expanded = true;
        }

        private void BindingToIListCollection()
        {
            //Remove all previous rows and headers
            ClearData();

            grid.Headers.Add(Header.FromDataType(typeof(ChildClass)));

            //Set header appearance
            SetAppearance();
            grid.Hierarchy.ButtonBehaviour = ExpansionButtonBehaviour.HideAlways;
            
            IList<ChildClass> list = new List<ChildClass>();

            Random random = new Random();
            for (int i = 0; i < 20; ++i)
            {
                list.Add(new ChildClass(DateTime.Now + new TimeSpan(1000000000000 * (random.Next() % 1000)),
                                        random.Next(5),
                                        new TimeSpan(random.Next())));
            }

            grid.DataSource = list;
        }

        private void BindingToXmlDataSource()
        {
            ClearData();

            //Read xml file from resources
            string ns = typeof (ExampleForm).FullName.Substring(0, typeof (ExampleForm).FullName.IndexOf(typeof (ExampleForm).Name));
            Stream str = typeof (ExampleForm).Assembly.GetManifestResourceStream(ns + "XmlDataSource.xml");

            //Get a data table
            DataSet dataSet = new DataSet();
            dataSet.ReadXml(str);
            DataTable table = dataSet.Tables["Employee"];

            grid.Hierarchy.ButtonBehaviour = ExpansionButtonBehaviour.HideAlways;
            grid.Headers.AutoGenerate = true;
            grid.DataSource = table;
            
            SetAppearance();

            grid.EditInPlace.Enabled = true;
        }

        private void BindingToDataSet()
        {
            ClearData();

            grid.Headers.AutoGenerate = true;
            grid.DataMember = "Orders";
            grid.DataSource = CreateDataSet();

            SetAppearance();
            grid.EditInPlace.Enabled = true;
        }

        #region Private routines

        private void SetAppearance()
        {
            Header header1 = grid.Headers.Count > 0 ? grid.Headers[0] : null;
            Header header2 = grid.Headers.Count > 1 ? grid.Headers[1] : null;
            if (header1 != null)
            {
                header1.Appearance.ColumnPanel = new Appearance(Color.Black, Color.FromArgb(211, 233, 254), true, Color.FromArgb(170, 211, 253), GradientDirection.Vertical);
                header1.Appearance.EvenRows = new Appearance(Color.Black, Color.AliceBlue, true, Color.FromArgb(214, 234, 254), GradientDirection.Vertical);
                header1.Appearance.OddRows = new Appearance(Color.Black, Color.AliceBlue, true, Color.FromArgb(214, 234, 254), GradientDirection.Vertical);
                header1.GroupingEnabled = true;
                header1.Appearance.GroupPanel = header1.Appearance.ColumnPanel;
                foreach (Column column in header1.Columns)
                {
                    column.Appearance.CellTextAlignment = ContentAlignment.MiddleCenter;
                    column.Width = Math.Max(120, column.Width);
                }
            }

            if (header2 != null)
            {
                header2.Appearance.ColumnPanel = new Appearance(SystemColors.ControlText, Color.FromArgb(248, 203, 120), true, Color.FromArgb(245, 186, 75), GradientDirection.Vertical);
                header2.Appearance.EvenRows = new Appearance(Color.Black, Color.FromArgb(253, 241, 219), true, Color.FromArgb(250, 240, 209), GradientDirection.Vertical);
                header2.Appearance.OddRows = new Appearance(Color.Black, Color.FromArgb(251, 227, 183), true, Color.FromArgb(249, 210, 138), GradientDirection.Vertical);
                header2.GroupingEnabled = true;
                header2.Appearance.GroupPanel = header2.Appearance.ColumnPanel;

                foreach (Column column in header2.Columns)
                {
                    column.Appearance.CellTextAlignment = ContentAlignment.MiddleCenter;
                    column.Width = Math.Max(120, column.Width);
                }
            }

            grid.EditInPlace.Enabled = false;
            grid.RowSelector.Visible = true;
            grid.RowSelector.Appearance = new Appearance(Color.Black, Color.FromArgb(211, 233, 254), true, Color.FromArgb(170, 211, 253), GradientDirection.Vertical);
        }

        private void ClearData()
        {
            grid.DataSource = null;
            grid.DataMember = null;
            grid.Rows.Clear();
            grid.Headers.Clear();
            grid.Headers.AutoGenerate = false;
        }

        public static DataSet CreateDataSet()
        {
            //Create some products
            IList<string> products = new List<string>();
            for (int i = 1; i < 10; ++i)
            {
                products.Add(string.Format("Product {0}", i));
            }

            // create a DataSet with one table, two columns
            DataSet dataSet = new DataSet();

            // create Customer table
            DataTable table = new DataTable("Orders");
            dataSet.Tables.Add(table);
            table.Columns.Add("orderId", typeof(int)).AutoIncrement = true;
            table.Columns.Add("product", typeof(string));
            table.Columns.Add("amount", typeof(decimal));
            table.Columns.Add("quantity", typeof(long));
            table.PrimaryKey = new DataColumn[] { table.Columns["orderId"] };

            //populate the table
            Random random = new Random();
            for (int orderId = 1; orderId <= 1000; orderId++)
            {
                // add customer record
                table.Rows.Add(new object[] { orderId, products[random.Next() % products.Count], Decimal.Round((decimal)random.Next(20, 100) / random.Next(1, 10), 3), (long)random.Next(1, 10) });
            }

            return dataSet;
        }

        #endregion

        #region Event handlers

        private void OnBindingListModeChanged(object sender, EventArgs e)
        {
            if (rbIBindingList.Checked)
            {
                BindingToDataSet();
            }
        }

        private void OnDataObjectModeChanged(object sender, EventArgs e)
        {
            if (rbDataObject.Checked)
            {
                AddingOneByOne();
            }
        }

        private void OnIListModeChanged(object sender, EventArgs e)
        {
            if (rbIList.Checked)
            {
                BindingToIListCollection();
            }
        }

        private void OnIDictionaryModeChanged(object sender, EventArgs e)
        {
            if (rbIDictionary.Checked)
            {
                AddingIDictionaryObjects();
            }
        }

        private void OnXmlDataSourceModeChanged(object sender, EventArgs e)
        {
            if (rbXmlDataSource.Checked)
            {
                BindingToXmlDataSource();
            }
        }

        #endregion
    }
}