using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using Dapfor.Net.Data;
using Dapfor.Net.Demo.Properties;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.CustomEditors
{
    public partial class CustomColorControl : UserControl
    {
        private readonly IWindowsFormsEditorService service;
        private Color editingColor;

        public CustomColorControl(IWindowsFormsEditorService service, Color editingColor)
            : this()
        {
            this.service = service;
            this.editingColor = editingColor;

            //Navigate to the initial color
            foreach (Row row in grid1.Rows)
            {
                ColorDataObject dataObject = (ColorDataObject) row.DataObject;
                if (dataObject.Color.R == editingColor.R && dataObject.Color.G == editingColor.G && dataObject.Color.B == editingColor.B)
                {
                    //Set initial opacity
                    dataObject.Opacity = editingColor.A;

                    grid1.Selection.Clear();
                    row.Selected = true;
                    row.Focused = true;
                    row.EnsureVisible();
                    break;
                }
            }
        }

        public CustomColorControl()
        {
            InitializeComponent();

            //Set special editors
            grid1.Headers[0]["opacity"].Editor = new CustomTrackBarEditor(255);
            grid1.Headers[0]["color"].Editor = (UITypeEditor) TypeDescriptor.GetEditor(typeof (Color), typeof (UITypeEditor));

            //Populate the grid
            foreach (KnownColor value in Enum.GetValues(typeof (KnownColor)))
            {
                Color color = Color.FromKnownColor(value);
                if (!color.IsSystemColor && !color.IsEmpty)
                {
                    grid1.Rows.Add(new ColorDataObject(color));
                }
            }
        }
        /// <summary>
        /// Gets or sets the color that is currently editing.
        /// </summary>
        /// <value>The color of the editing.</value>
        public Color EditingColor
        {
            get { return editingColor; }
            set { editingColor = value; }
        }

        private void OnPaintCell(object sender, PaintCellEventArgs e)
        {
            //Customize the painting. 
            if (e.Cell.Row != null && e.Cell.Column != null && e.Cell.Column.Id == "select")
            {
                //Draw the 'Select' button
                Point pt = grid1.PointToClient(Cursor.Position);
                Image image = e.Cell.VirtualBounds.Contains(pt) ? Resources.check : Resources.bullet_square_grey;
                e.Image = image;
                e.ImageSettings.Alignment = ContentAlignment.MiddleCenter;
            }
        }

        private void OnCellEnterOrLeave(object sender, GridCellEventArgs e)
        {
            //Invalidate the 'Select' button
            if (e.Cell.Row != null && e.Cell.Column != null && e.Cell.Column.Id == "select")
            {
                e.Cell.Invalidate();
            }
        }

        private void OnKeyDown(object sender, KeyEventArgs e)
        {
            //Close the control when the user tapes the 'Enter' or 'Escape' keys
            if (service != null)
            {
                switch (e.KeyCode)
                {
                    case Keys.Enter:
                        ApplyColor(grid1.FocusedRow);
                        service.CloseDropDown();
                        break;

                    case Keys.Escape:
                        service.CloseDropDown();
                        break;
                }
            }
        }

        private void OnCellClick(object sender, GridCellEventArgs e)
        {
            //Focus and select the row
            if (e.Cell.Row != null)
            {
                e.Cell.Row.Grid.Selection.Clear();
                e.Cell.Row.Selected = true;
                e.Cell.Row.Focused = true;
            }

            //If user has clicked on the 'Select' button, we will close the control
            if (e.Cell.Row != null && e.Cell.Column != null && e.Cell.Column.Id == "select")
            {
                if (service != null)
                {
                    ApplyColor(e.Cell.Row);
                    service.CloseDropDown();
                }
            }
        }

        private void ApplyColor(Row row)
        {
            if (row != null)
            {
                int transparency = (int) row["opacity"].Value;
                Color color = (Color) row["color"].Value;

                //Apply the transparency to the selected color
                color = Color.FromArgb(transparency, color);

                //Set a new color to the data object
                editingColor = color;
            }
        }

        private void OnDblClick(object sender, EventArgs e)
        {
            //Close the control
            ApplyColor(grid1.FocusedRow);
            service.CloseDropDown();
        }

        #region Nested type: ColorDataObject

        //Data object to be inserted to the grid
        public class ColorDataObject
        {
            private readonly Color color;
            private int opacity;

            public ColorDataObject(Color color)
            {
                this.color = color;
                opacity = color.A;
            }

            [Field("opacity")]
            public int Opacity
            {
                get { return opacity; }
                set { opacity = value; }
            }

            [Field("color")]
            public Color Color
            {
                get { return color; }
            }
        }

        #endregion
    }
}