using System;
using System.Collections.Generic;

namespace Dapfor.Net.Demo.Examples.Miscellaneous.CurrencyViewer
{
    public enum CurrencyType
    {
        USD,
        EUR,
        CAD,
        GBP,
        JPY,
        AUD,
        ZAR,
        SEK,
        CHF,
        DKK,
        HKD,
        INR,
        ILS,
        ISK,
        MAD,
        RUB,
    }

    /// <summary>
    /// Currency rate between _mainCurrency and currencies, stored in the _rates container
    /// </summary>
    public class CurrencyRate
    {
        private readonly CurrencyType _mainCurrency;
        private readonly Dictionary<CurrencyType, double> _prevRates = new Dictionary<CurrencyType, double>();
        private readonly Dictionary<CurrencyType, double> _rates = new Dictionary<CurrencyType, double>();

        public CurrencyRate(CurrencyType mainCurrency)
        {
            _mainCurrency = mainCurrency;
            foreach (CurrencyType currency in Enum.GetValues(typeof (CurrencyType)))
            {
                _rates.Add(currency, 1);
            }
            foreach (CurrencyType currency in Enum.GetValues(typeof (CurrencyType)))
            {
                _prevRates.Add(currency, 1);
            }
        }

        public RateInfo this[CurrencyType currency]
        {
            get { return new RateInfo(currency, this); }
        }

        public CurrencyType MainCurrency
        {
            get { return _mainCurrency; }
        }

        public event EventHandler<RateChangedEventArgs> RateChanged;

        #region Nested type: RateInfo

        /// <summary>
        /// Rate between _mainCurrency and one of currencies, stored in the _rates container
        /// </summary>
        public class RateInfo
        {
            private readonly CurrencyType _currency2;
            private readonly CurrencyRate _owner;


            public RateInfo(CurrencyType currency2, CurrencyRate owner)
            {
                _currency2 = currency2;
                _owner = owner;
            }

            public CurrencyType Currency1
            {
                get { return _owner._mainCurrency; }
            }

            public CurrencyType Currency2
            {
                get { return _currency2; }
            }

            public double PrevRate
            {
                get { return _owner._prevRates[_currency2]; }
            }

            public double Rate
            {
                get { return _owner._rates[_currency2]; }
                set
                {
                    _owner._prevRates[_currency2] = _owner._rates[_currency2];
                    _owner._rates[_currency2] = value;
                    if (_owner.RateChanged != null)
                    {
                        _owner.RateChanged(_owner, new RateChangedEventArgs(_owner, _currency2, value));
                    }
                }
            }
        }

        #endregion
    }

    public sealed class RateChangedEventArgs : EventArgs
    {
        private readonly CurrencyType _currency;
        private readonly CurrencyRate _currencyRate;
        private readonly double _rate;

        public RateChangedEventArgs(CurrencyRate currencyRate, CurrencyType currency, double rate)
        {
            _currencyRate = currencyRate;
            _currency = currency;
            _rate = rate;
        }

        public CurrencyRate CurrencyRate
        {
            get { return _currencyRate; }
        }

        public CurrencyType Currency
        {
            get { return _currency; }
        }

        public double Rate
        {
            get { return _rate; }
        }
    }
}