using System;
using System.Drawing;
using System.IO;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Miscellaneous.FileExplorer
{
    [Example("File explorer", "Miscellaneous")]
    public partial class FileExplorerDemoForm : UserControl
    {
        public FileExplorerDemoForm()
        {
            InitializeComponent();


            // Store in a string array
            string[] drives = Environment.GetLogicalDrives();
            // Loop into the string array
            foreach (string strDrive in drives)
            {
                Row row = grid.Rows.Add(new DriveObject(strDrive));
                
                //Expand the drive C:
                if(string.Compare("c:\\", strDrive, true) == 0)
                {
                    row.Expanded = true;
                }
            }
        }

        #region Private event handlers

        private void OnPaintCell(object sender, PaintCellEventArgs e)
        {
            //Paint an image in the cell
            if (e.Cell.Column != null && e.Cell.Row != null && e.Cell.Column.Id == "Name")
            {
                IExplorable dir = (IExplorable) e.Cell.Row.DataObject;
                using (Image image = dir != null ? GetIcon(dir.FullName) : null)
                {
                    e.Image = image;
                    e.ImageSettings.Alignment = ContentAlignment.MiddleLeft;
                    e.TextSettings.Padding = new Padding(6, 0, 0, 0);

                    e.PaintAll();
                    e.Handled = true;
                }
            }
        }

        private void OnPaintHierarchy(object sender, PaintHierarchyEventArgs e)
        {
            IExplorable dir = e.Row != null ? (IExplorable)e.Row.DataObject : null;
            if (dir != null)
            {
                if(dir.Expandable)
                {
                    //Paint +/- buttons
                    e.Parts |= PaintPart.ExpansionButton;
                }
                else
                {
                    //Prevent from +/- buttons painting
                    e.Parts &= e.Parts ^ PaintPart.ExpansionButton;
                }
            }
        }

        private void OnRowExpansionChanged(object sender, GridRowEventArgs e)
        {
            //When the row is opened, we can add files and directories as subitems of this row
            IExplorable dir = e.Row != null ? e.Row.DataObject as IExplorable : null;
            if (dir != null && e.Row.Children.Count == 0)
            {
                DirectoryInfo di = new DirectoryInfo(dir.FullName);
                if (di.Exists)
                {
                    try
                    {
                        foreach (DirectoryInfo directoryInfo in di.GetDirectories("*", SearchOption.TopDirectoryOnly))
                        {
                            //Add a new subfolder
                            e.Row.Add(new DirectoryObject(directoryInfo.Name, dir));
                        }

                        foreach (FileInfo fileInfo in di.GetFiles("*", SearchOption.TopDirectoryOnly))
                        {
                            //Add a file
                            e.Row.Add(new FileObject(fileInfo.Name, fileInfo.CreationTime, fileInfo.Length, fileInfo.Attributes, dir));
                        }
                    }
                    catch
                    {
                        //Access is denied
                    }
                }
            }
        }

        #endregion

        #region Private methods

        private static Image GetIcon(string path)
        {
            //Extracts icon from the file
            Image image = null;
            NativeMethods.SHFILEINFO shinfo = new NativeMethods.SHFILEINFO();

            // Get a handle to the small icon
            IntPtr hImgSmall = NativeMethods.SHGetFileInfo(path, 0, ref shinfo, Marshal.SizeOf(shinfo), NativeMethods.SHGFI_ICON | NativeMethods.SHGFI_SMALLICON);
            if (hImgSmall.ToInt32() != 0 && shinfo.hIcon.ToInt32() != 0)
            {
                using(Icon myIcon = Icon.FromHandle(shinfo.hIcon))
                {
                    image = myIcon.ToBitmap();
                    NativeMethods.DestroyIcon(shinfo.hIcon);
                }
            }

            return image;
        }

        #endregion
    }
}