using System;
using System.Drawing;
using System.Globalization;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Demo.Properties;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Performance.HierarchicalUpdates
{
    [Example("Hierarchical grid updating", "Performance")]
    public partial class HierarchicalUpdatesDemoForm : UserControl
    {
        #region Delegates

        public delegate void CallBack();

        #endregion

        #region Fields

        private Timer _guiTimer;
        private DateTime _lastTimeStamp = DateTime.MinValue;
        private Random _random = new Random();
        private int _updatesPerSecond;

        #endregion

        public HierarchicalUpdatesDemoForm()
        {
            //Initialize the grid
            InitializeComponent();
            
            lbGridSize.Text = string.Empty;

            //Start Gui timer
            _guiTimer = new Timer();
            _guiTimer.Interval = 10;
            _guiTimer.Tick += OnGuiTimerTick;

            tbRowCount.Text = "2000";
            tbUpdatePerSec.Text = "5000";

            //Expand some rows
            for(int i = 0; i < grid.Nodes.Count; ++i)
            {
                if((i + 4) % 7 == 0)
                {
                    grid.Nodes[i].Expanded = true;    
                }
            }

            lbVisibleRows.Text = grid.Rows.Count.ToString();
        }

        #region Private event handlers

        private void OnRowCountChanged(object sender, EventArgs e)
        {
            int count;
            if (!string.IsNullOrEmpty(tbRowCount.Text) && int.TryParse(tbRowCount.Text, NumberStyles.Number, CultureInfo.InvariantCulture, out count) && count >= 0)
            {
                //Modify count of dataobjects in the dataObjects container and in the grid
                if (grid.Nodes.Count < count)
                {
                    //Add new items to the Grid and dataObjects container
                    long itemsToAdd = count - grid.Nodes.Count;
                    for (int i = 0; i < itemsToAdd; ++i)
                    {
                        RootDataObject rootObject = new RootDataObject(String.Format("Node {0}: To expand the node double click on the row.", (object) grid.Nodes.Count));
                        Row root = grid.Rows.Add(rootObject);
                        for(int j = 0; j < 10; ++j)
                        {
                            root.Add(new DataObject(_random));
                        }
                    }
                }
                else if (count < grid.Nodes.Count)
                {
                    //Create a list of items to be removed
                    int toRemove = grid.Nodes.Count - count;

                    for (int i = 0; i < toRemove; ++i)
                    {
                        grid.Rows.RemoveAt(grid.Rows.Count - 1);
                    }
                }

                //Adjust the first column to the best size
                grid.Headers[0][0].Adjust(true, false);
            
            }
            else if (string.IsNullOrEmpty(tbRowCount.Text))
            {
                grid.Rows.Clear();
            }

            lbVisibleRows.Text = grid.Rows.Count.ToString();
        }

        private void OnGuiTimerTick(object sender, EventArgs e)
        {
            DateTime current = DateTime.UtcNow;
            TimeSpan ts = current - _lastTimeStamp;

            //Update visible cells 
            if (_updatesPerSecond > 0 && grid.Rows.Count > 0)
            {
                //Compute a count of updates for the current timer's tick
                int updatesCount = (int)((ts.TotalMilliseconds * _updatesPerSecond) / 1000);
                updatesCount = Math.Max(1, updatesCount);

                for (int i = 0; i < updatesCount; ++i)
                {
                    //Root item can be of the next types: RootDataObject or DataObject. Update only the DataObject
                    Row row = grid.Rows[GetRandom(grid.Rows.Count)];
                    DataObject dataObject = row.DataObject as DataObject;
                    if (dataObject == null)
                    {
                        //Look for a data object
                        row = row.FirstVisibleChild;
                        if (row != null)
                        {
                            dataObject = row.DataObject as DataObject;
                        }
                    }

                    //If the data object exists, update it.
                    if (dataObject != null)
                    {
                        UpdateDataObject(dataObject);
                    }
                }
            }

            _lastTimeStamp = DateTime.UtcNow;
        }

        private void OnUpdatesCountChanged(object sender, EventArgs e)
        {
            //Set a new amount of updates per second
            if (!string.IsNullOrEmpty(tbUpdatePerSec.Text))
            {
                int count;
                if (int.TryParse(tbUpdatePerSec.Text, out count))
                {
                    _updatesPerSecond = count;
                }
            }
            else
            {
                _updatesPerSecond = 0;
            }
            InitializeTimers();
        }

        private void OnGridSizeChanged(object sender, EventArgs e)
        {
            lbGridSize.Text = string.Format("{0} x {1}", grid.Size.Width, grid.Size.Height);
        }

        private void OnGridCellClicked(object sender, GridCellEventArgs e)
        {
            //Expand or collapse row when the end-user clicks on the cell with 'expansion' image
            if (e.Cell.Row != null && e.Cell.Column != null && e.Cell.Row.Level == 0 && e.Cell.Column.Id == "_id0")
            {
                e.Cell.Row.Expanded = !e.Cell.Row.Expanded;
            }
        }

        private void OnGridPaintCell(object sender, PaintCellEventArgs e)
        {
            //Set expand/collapse icon
            if(e.Cell.Row != null && e.Cell.Column != null && e.Cell.Row.Level == 0 && e.Cell.Column.Id == "_id0")
            {
                e.Image = e.Cell.Row.Expanded ? Resources.nav_up_red : Resources.nav_redo_green;
                e.ImageSettings.Alignment = ContentAlignment.MiddleCenter;
            }
        }

        private void OnExpandAllRows(object sender, EventArgs e)
        {
            grid.Rows.ExpandAll();
            lbVisibleRows.Text = grid.Rows.Count.ToString();
        }

        private void OnCollapseAllRows(object sender, EventArgs e)
        {
            grid.Rows.CollapseAll();
            lbVisibleRows.Text = grid.Rows.Count.ToString();
        }

        #endregion

        #region Private methods

        private void UpdateDataObject(DataObject dataObject)
        {
            //Set a random value into a random field of the data object
            switch (GetRandom(10))
            {
                case 0:
                    dataObject.Value0 = GetRandom(0x7FFFFFFF);
                    break;
                case 1:
                    dataObject.Value1 = GetRandom(0x7FFFFFFF);
                    break;
                case 2:
                    dataObject.Value2 = GetRandom(0x7FFFFFFF);
                    break;
                case 3:
                    dataObject.Value3 = GetRandom(0x7FFFFFFF);
                    break;
                case 4:
                    dataObject.Value4 = GetRandom(0x7FFFFFFF);
                    break;
                case 5:
                    dataObject.Value5 = GetRandom(0x7FFFFFFF);
                    break;
                case 6:
                    dataObject.Value6 = GetRandom(0x7FFFFFFF);
                    break;
                case 7:
                    dataObject.Value7 = GetRandom(0x7FFFFFFF);
                    break;
                case 8:
                    dataObject.Value8 = GetRandom(0x7FFFFFFF);
                    break;
                case 9:
                    dataObject.Value9 = GetRandom(0x7FFFFFFF);
                    break;
            }
        }

        private void InitializeTimers()
        {
            int interval = _updatesPerSecond > 0 ? 1000 / _updatesPerSecond : 10;
            interval = Math.Min(1000, interval);
            interval = Math.Max(10, interval);


            _lastTimeStamp = DateTime.UtcNow;

            //Start a GUI timer and set interval
            _guiTimer.Interval = interval;
            _guiTimer.Start();

        }

        private int GetRandom(int max)
        {
            //Gets a random integer value
            int result = _random.Next();
            if (result == 0)
            {
                _random = new Random();
            }
            return max > 0 ? result % max : 0;
        }

        #endregion
    }
}